@@ -1,13 +1,11 @@
 package Class::Easy::Log;
 # $Id: Log.pm,v 1.3 2009/07/20 18:00:10 apla Exp $
 
-use strict;
-use warnings;
-
-require Class::Easy;
-require Class::Easy::Timer;
-
+use Class::Easy::Import;
 use Class::Easy::Log::Tie;
+use Class::Easy ();
+
+
 
 # log4perl has categories, layouts and appenders
 our $default_layout = '[%P] [%M(%L)] [%c] %m%n';
@@ -73,9 +71,19 @@ sub logger { # create logger
 	
 	my $driver_id;
 	my $category;
+	my $appender;
+	
+	my $ref;
+	
+	if (defined $_[1]) {
+		$ref = ref \$_[1];
+	}
 	
 	unless (@_) { # if type omitted, we use current package name as type
 		$category = (caller)[0];
+	} elsif (scalar (@_) == 2 and $ref eq 'GLOB' and defined *{$_[1]}{IO}) {
+		$category = $_[0];
+		$appender = $_[1];
 	} elsif ((@_ == 2 or @_ == 1) and exists $driver_config->{$_[0]}) {
 		$driver_id = $_[0];
 		$category = @_ == 1 ? (caller)[0] : $_[1];
@@ -89,24 +97,28 @@ sub logger { # create logger
 	
 	unless (defined $driver_id) { # basic internal driver require no processing
 		
-		$self = $int_loggers->{$category} || bless {
+		my $existing_logger = $int_loggers->{$category};
+		
+		$self = $existing_logger || bless {
 			category => $category,
 			broker   => '',
 		}, 'Class::Easy::Log';
 		
-		$int_loggers->{$category} = $self;
-		
-		Class::Easy::make_accessor ((caller)[0], 'log_'.$category, default => sub {
-			my $caller1  = [caller (1)];
-			my $caller0  = [caller];
+		unless (defined $existing_logger) {
+			$int_loggers->{$category} = $self;
+			
+			Class::Easy::make_accessor ((caller)[0], 'log_'.$category, default => sub {
+				my $caller1  = [caller (1)];
+				my $caller0  = [caller];
 
-			unshift @_, $category, $self, $caller1, $caller0;
-			goto &_wrapper;
-		});
+				unshift @_, $category, $self, $caller1, $caller0;
+				goto &_wrapper;
+			});
 
-		Class::Easy::make_accessor ((caller)[0], 'timer_'.$category, default => sub {
-			Class::Easy::Timer->new (@_, $self)
-		});
+			Class::Easy::make_accessor ((caller)[0], 'timer_'.$category, default => sub {
+				Class::Easy::Timer->new (@_, $self)
+			});
+		}
 
 	} elsif (defined $driver_config->{$driver_id}) { # driver defined
 		my $driver = $driver_config->{$driver_id};
@@ -119,6 +131,10 @@ sub logger { # create logger
 		# make_accessor ((caller)[0], 'log_'.$type, default => \&Class::Easy::Log::message);
 	}
 	
+	if ($appender) {
+		$self->appender ($appender);
+	}
+	
 	return $self;
 }
 
@@ -144,6 +160,11 @@ sub appender {
 sub _parse_layout {
 	my $logger = shift;
 	
+	$logger->{layout} ||= $default_layout;
+	
+	return $logger
+		if defined $logger->{_layout} and $logger->{layout} eq $logger->{_layout};
+	
 	my $layout = $logger->{layout};
 	
 	my $layout_format = '';
@@ -170,7 +191,7 @@ sub _parse_layout {
 	$logger->{_layout_fields} = \@layout_fields;
 	$logger->{_layout} = $layout;
 	
-	return 1;
+	return $logger;
 }
 
 sub _format_log {
@@ -201,9 +222,14 @@ sub _format_log {
 #	warn Dumper $self->{_layout_fields};
 #	warn Dumper [map {$values->{$_}} @{$self->{_layout_fields}}];
 	
+#	warn $self->{_layout_format}, join (', ', @{$self->{_layout_fields}}), (join ', ', map {
+#		$values->{$_}
+#	} @{$self->{_layout_fields}});
+	
 	return sprintf ($self->{_layout_format}, (map {
 		$values->{$_}
 	} @{$self->{_layout_fields}}));
+	
 }
 
 sub _wrapper {
@@ -218,14 +244,7 @@ sub _wrapper {
 	# my ($package, $filename, $line, $subroutine, $hasargs, $wantarray,
 	# $evaltext, $is_require, $hints, $bitmask)
 	
-	$logger->{layout} ||= $default_layout;
-	
-	if (defined $logger->{_layout} and $logger->{_layout} eq $logger->{layout}) {
-		
-	} else {
-		# parse layout
-		$logger->_parse_layout;
-	}
+	$logger->_parse_layout;
 		
 	$logger->{broker} = $logger->_format_log (
 		message => join ('', @_),
@@ -255,14 +274,25 @@ sub debug_depth {
 }
 
 sub critical {
-	my $message  = join '', @_ ;
-
 	my $sub  = (caller (1))[3] || 'main';
 	my $line = (caller)[2];
-
-	&$Class::Easy::LOGGER ($message);
-	die "[$$] [$sub($line)] [DIE] $message\n";
+	
+	my $logger = logger ('DIE')->_parse_layout;
+	
+	die $logger->_format_log (
+		message => join ('', @_),
+		method  => $sub,
+		line    => $line
+	);
 }
 
+sub catch_stderr {
+	my $ref = shift;
+	tie *STDERR => 'Class::Easy::Log::Tie', $ref;
+}
+
+sub release_stderr {
+	untie *STDERR;
+}
 
 1;
\ No newline at end of file
